@@ -102,26 +102,66 @@ class PipelineGraph(object):
 
         return tasks, None if max_parallelism < 1 else max_parallelism, slice
 
-    def get_last_task_in_chain(self, task_id):
-        task_chain = self.get_task_chain(task_id)
-        return task_chain[-1]
+    def get_last_tasks_in_chain(self, task_id):
+        """
+        Returns the last tasks in the chain that this task_id is part of
+        
+        Examples:
+        
+        a -> [a, b, c] => [c]
+        b -> [a, b, c] => [c]
 
-    def mark_task_complete(self, task_id, task_result_or_exception):
+        Exceptionally tasks are included but only if they form the remainder of the chain
+
+        Examples:
+        b -> [a, b, c, ex(d)] => [c, d]
+        b -> [a, b, c, ex(d), ex(e)] => [c, d, e]
+        b -> [a, b, c, ex(d), ex(e), f] => [f]
+        """
+        task_chain = self.get_task_chain(task_id)
+
+        tasks = []
+
+        for task in reversed(task_chain):
+            tasks.insert(0, task)
+
+            if not task.is_exceptionally:
+                break
+            
+        return tasks
+
+    def mark_task_complete(self, task_id, task_result_or_exception, remark=False):
         failed = isinstance(task_result_or_exception, TaskException)
 
         tasks = self.get_task_chain(task_id)
 
+        if remark:
+            for task in tasks:
+                task.mark_complete(False)
+
         if any(tasks):
             if tasks[0].is_complete():
                 return False  # task already complete - ignore
 
+            tasks[0].mark_complete()
+
             if failed:
-                # mark this task complete and its children
+                # mark mark subsequent tasks complete until we reach the first exceptionally continuation
+                # this is where we will continue from
                 for task in tasks:
-                    if not task.is_finally:
+                    if task.is_exceptionally:
+                        break  
+
+                    elif not task.is_finally:
                         task.mark_complete()
             else:
-                tasks[0].mark_complete()
+                # mark subsequent exceptionally tasks complete until we reach the first non-exceptionally continuation
+                # this is where we will continue from
+                for task in tasks[1:]:
+                    if not task.is_exceptionally:
+                        break
+
+                    task.mark_complete()
 
         return True
          
@@ -133,7 +173,44 @@ class PipelineGraph(object):
         finally_task = next((task for task in self._pipeline if isinstance(task, Task) and task.is_finally), None)
         return finally_task is not None and finally_task.uid == task_id
 
-    def get_next_step_in_pipeline(self, task_id):
+    def get_next_step_in_pipeline(self, task_id, task_result_or_exception):
+        current_step, next_step, group, empty_group = self._get_next_step_in_pipeline(task_id)
+
+        skipped_tasks = []
+
+        if isinstance(task_result_or_exception, TaskException):
+            # if we have a TaskException as input then we need to skip forward to the next exceptionally
+            # task if we have one or otherwise to the finally task if we have one
+            task = next_step
+
+            while task is not None and isinstance(task, Task) and not task.is_exceptionally:
+                skipped_tasks.append(next_step)
+                _, task, _, _ = self._get_next_step_in_pipeline(task.uid)
+
+            next_step = task or next_step
+
+            if not (isinstance(next_step, Task) and next_step.is_exceptionally):
+                next_step = None
+
+                # if the task that failed is part of a group then wait for the group to complete before triggering the finally
+                if group is not None:
+                    if group.is_complete():
+                        next_step = self.try_get_finally_task(task_id)
+
+                else:
+                    next_step = self.try_get_finally_task(task_id)
+
+        else:
+            # otherwise we have a TaskResult and so we need to skip over exceptionally tasks
+            while next_step is not None and isinstance(next_step, Task) and next_step.is_exceptionally:
+                next_step.mark_complete()
+                skipped_tasks.append(next_step)
+
+                _, next_step, _, _ = self._get_next_step_in_pipeline(next_step.uid)
+
+        return current_step, next_step, group, empty_group, skipped_tasks
+
+    def _get_next_step_in_pipeline(self, task_id):
         stack = deque([(self._pipeline, None, None, None, None)])  # FIFO (pipeline, entry after group, group, parent_group, entry after parent group)
 
         while len(stack) > 0:
@@ -173,7 +250,7 @@ class PipelineGraph(object):
                         while isinstance(next_entry, Group) and not any(next_entry):
                             empty_group_entry = next_entry
                             next_entry = _try_next(iterator)
-                            
+                        
                         return entry, next_entry, group_entry, empty_group_entry
 
         # entry, next_entry, group entry is part of, next_entry group skipped over because it was empty
