@@ -22,16 +22,16 @@ class DeferredTaskSubmitter(object):
         context: TaskContext, 
         tasks: List[Task], 
         task_result_or_exception=None, 
-        task_state=None, 
         initial_args=None, 
-        initial_kwargs=None, 
+        initial_kwargs=None,
+        initial_state=None,
         max_parallelism=None):     
         
         # unpack task_result into result + state so we can merge in extras later
         if task_result_or_exception is not None:
             task_result, task_state = self._serialiser.unpack_response(task_result_or_exception)
         else:
-            task_result, task_state = None, task_state
+            task_result, task_state = None, None
 
         # split into tasks to call vs those to defer
         if max_parallelism is None or max_parallelism < 1:
@@ -40,7 +40,7 @@ class DeferredTaskSubmitter(object):
             tasks_to_call, tasks_to_defer = tasks[:max_parallelism], tasks[max_parallelism:]
 
         if any(tasks_to_defer):
-            has_initial_args = initial_args is not None or initial_kwargs is not None
+            has_initial_args = initial_args is not None or initial_kwargs is not None or initial_state is not None
             
             # create task deferral to hold these deferred tasks
             deferral_id = self._create_task_deferral(context, tasks_to_defer, task_result_or_exception, has_initial_args)
@@ -48,23 +48,24 @@ class DeferredTaskSubmitter(object):
             # parent tasks for the deferrals are the final tasks in each chain beginning with the tasks we are going to call
             # when a parent task completes, a task from the deferral will be called keep the overall parallism at max_parallelism
             for task in tasks_to_call:
-                last_task = self._graph.get_last_task_in_chain(task.uid)
-                context.pipeline_state.task_deferral_ids_by_task_uid[last_task.uid] = deferral_id
-            
+                self._add_deferral_trigger(context, task, deferral_id)
+        
         for task in tasks_to_call:      
-            task_request = self._create_task_request(context, task, task.request, task_state, initial_args, initial_kwargs, task_result)
+            task_request = self._create_task_request(context, task, task.request, task_state, initial_args, initial_kwargs, initial_state, task_result)
             self._send_task(context, task.get_destination(), task_request)
 
 
     def release_tasks(self, context, caller_uid, task_result_or_exception):
+        deferral_triggers = self._get_deferral_triggers(caller_uid, task_result_or_exception)
         
-        if isinstance(task_result_or_exception, TaskException):
-            parent_id = self._graph.get_last_task_in_chain(caller_uid).uid
-        else:
-            parent_id = caller_uid
-        
-        if parent_id in context.pipeline_state.task_deferral_ids_by_task_uid:
-            deferral_id = context.pipeline_state.task_deferral_ids_by_task_uid[parent_id]
+        if not any(deferral_triggers):
+            return
+
+        # process trigger (we only use one and delete the rest below)
+        deferral_trigger = deferral_triggers[-1]
+
+        if deferral_trigger in context.pipeline_state.task_deferral_ids_by_task_uid:
+            deferral_id = context.pipeline_state.task_deferral_ids_by_task_uid[deferral_trigger]
             task_deferral = context.pipeline_state.task_deferrals_by_id[deferral_id]
 
             if any(task_deferral.tasks):
@@ -72,32 +73,64 @@ class DeferredTaskSubmitter(object):
                 deferred_task = task_deferral.tasks.pop()
                 task = self._graph.get_task(deferred_task.task_uid) 
 
-                initial_args, initial_kwargs = None, None
+                initial_args, initial_kwargs, initial_state = None, None, None
 
-                if deferred_task.has_initial_args:
-                    initial_args = unpack_any(context.pipeline_state.pipeline.initial_args, known_proto_types=[])
-                    initial_kwargs = context.pipeline_state.pipeline.initial_kwargs
+                if deferred_task.has_initial_args:         
+                    # send initial state to each initial task if we have some
+                    if context.pipeline_state.pipeline.HasField('initial_state'):
+                        initial_state = unpack_any(context.pipeline_state.pipeline.initial_state, known_proto_types=[])
 
-                task_request = self._create_task_request_from_deferral(context, task, task_deferral, initial_args, initial_kwargs)
+                    # or the current task state if this pipeline is inline
+                    elif context.pipeline_state.pipeline.inline:
+                        initial_state = unpack_any(context.pipeline_state.task_state, known_proto_types=[])
+
+                    # send initial arguments to each initial task if we have them
+                    if context.pipeline_state.pipeline.HasField('initial_args'):
+                        initial_args = unpack_any(context.pipeline_state.pipeline.initial_args, known_proto_types=[])
+
+                    # send initial kwargs to each initial task if we have them
+                    if context.pipeline_state.pipeline.HasField('initial_kwargs'):
+                        initial_kwargs = context.pipeline_state.pipeline.initial_kwargs
+
+                task_request = self._create_task_request_from_deferral(context, task, task_deferral, initial_args, initial_kwargs, initial_state)
                 
-                # add task we are submitting to deferral task
-                last_task = self._graph.get_last_task_in_chain(deferred_task.task_uid)
-
-                context.pipeline_state.task_deferral_ids_by_task_uid[last_task.uid] = deferral_id
-
+                # add task we are submitting to deferral trigger so that when it completes we can release a further deferred task
+                self._add_deferral_trigger(context, task, deferral_id)
+            
                 # send the task
                 self._send_task(context, task.get_destination(), task_request)
             else:
-                # clean up
-                del context.pipeline_state.task_deferral_ids_by_task_uid[parent_id]
+                # delete deferral now that it is empty
                 del context.pipeline_state.task_deferrals_by_id[deferral_id]
 
+            # clean up used deferral triggers
+            for task_id in deferral_triggers:
+                if task_id in context.pipeline_state.task_deferral_ids_by_task_uid:
+                    del context.pipeline_state.task_deferral_ids_by_task_uid[task_id]
+
     def unpause_tasks(self, context):
         for paused_task in context.pipeline_state.paused_tasks:
             context.send_message(paused_task.destination, paused_task.task_request.id, paused_task.task_request)
 
         context.pipeline_state.ClearField('paused_tasks')
 
+    def _add_deferral_trigger(self, context, task, deferral_id):
+        for task in self._graph.get_last_tasks_in_chain(task.uid):
+            context.pipeline_state.task_deferral_ids_by_task_uid[task.uid] = deferral_id
+
+    def _get_deferral_triggers(self, task_id, task_result_or_exception):
+        if isinstance(task_result_or_exception, TaskException):
+            # only return the triggers once we get to the last exception in the chain
+            task_ids = [task.uid for task in self._graph.get_last_tasks_in_chain(task_id) if task is not None]
+
+            if any(task_ids) and task_id == task_ids[-1]:
+                return task_ids
+            else:
+                return []
+        else:
+            # else return this task as the trigger
+            return [task_id]
+
     def _create_task_deferral(self, context, tasks_to_defer, task_result_or_exception, has_initial_args):
         # create task deferral to hold these deferred tasks
         deferral_id = _gen_id()
@@ -119,8 +152,9 @@ class DeferredTaskSubmitter(object):
 
         return deferral_id
 
-    def _create_task_request_from_deferral(self, context, task, task_deferral, initial_args, initial_kwargs):
+    def _create_task_request_from_deferral(self, context, task, task_deferral, initial_args, initial_kwargs, initial_state):
         task_state, task_result = None, None
+
         if task_deferral.HasField('task_result_or_exception'):
             
             if task_deferral.task_result_or_exception.HasField('task_result'):
@@ -130,9 +164,9 @@ class DeferredTaskSubmitter(object):
 
             task_result, task_state = self._serialiser.unpack_response(task_result_or_exception)
 
-        return self._create_task_request(context, task, task.request, task_state, initial_args, initial_kwargs, task_result, task_deferral)
+        return self._create_task_request(context, task, task.request, task_state, initial_args, initial_kwargs, initial_state, task_result, task_deferral)
 
-    def _create_task_request(self, context, task, task_request, task_state=None, initial_args=None, initial_kwargs=None, task_result=None, deferral=None):
+    def _create_task_request(self, context, task, task_request, task_state=None, initial_args=None, initial_kwargs=None, initial_state=None, task_result=None, deferral=None):
         if initial_args is not None or initial_kwargs is not None:
             task_args_and_kwargs = self._serialiser.to_args_and_kwargs(task.request)
             task.request = self._serialiser.merge_args_and_kwargs(task_args_and_kwargs, initial_args, initial_kwargs)
@@ -141,6 +175,9 @@ class DeferredTaskSubmitter(object):
             task_args_and_kwargs = self._serialiser.to_args_and_kwargs(task.request)
             task_request = self._serialiser.merge_args_and_kwargs(task_args_and_kwargs, task_result if not task.is_finally else TupleOfAny(), None)
 
+        # use initial state if provided
+        task_state = initial_state if initial_state is not None else task_state
+
         request = TaskRequest(id=task.task_id, type=task.task_type, uid=task.uid)
 
         # allow callers to drop the results of fruitful tasks in their pipelines if they wish
