@@ -1,7 +1,7 @@
 from statefun_tasks.serialisation import DefaultSerialiser
 from statefun_tasks.pipeline_builder import PipelineBuilder
 
-from statefun_tasks.types import Task, RetryPolicy, \
+from statefun_tasks.types import Task, RetryPolicy, MessageSizeExceeded, \
     TASK_STATE_TYPE, TASK_REQUEST_TYPE, TASK_RESULT_TYPE, TASK_EXCEPTION_TYPE, PIPELINE_STATE_TYPE
 
 from statefun_tasks.messages_pb2 import TaskResult, TaskException, TaskRequest, Address
@@ -31,23 +31,26 @@ class FlinkTasks(object):
     :param default_namespace: namespace to expose functions under. Maps to Flink Statefun function namespace in module.yaml
     :param default_worker_name: worker name to expose.  Maps to Flink Statefun function type in module.yaml
     :param egress_type_name: egress type name.  Maps to Flink Statefun egress in module.yaml
+    :param optional egress_message_max_size: maximum size of an egress message in bytes. If specified attempts to send messages over this size will raise a MessageSizeExceeded exception
     :param optional serialiser: serialiser to use (will use DefaultSerialiser if not set)
+    :param optional state_expiration: duration after which state will be expired by Flink (expire_after_call)
     """
-    def __init__(self, default_namespace: str = None, default_worker_name: str = None, egress_type_name: str = None, serialiser = None):
+    def __init__(self, default_namespace: str = None, default_worker_name: str = None, egress_type_name: str = None, egress_message_max_size: int = None, serialiser = None, state_expiration: timedelta = None):
         self._default_namespace = default_namespace
         self._default_worker_name = default_worker_name
         self._egress_type_name = egress_type_name
+        self._egress_message_max_size = egress_message_max_size
         self._serialiser = serialiser if serialiser is not None else DefaultSerialiser()
         self._bindings = {}
         self._events = EventHandlers()
         
         # to register in Flink's @functions.bind() attribute
         self._value_specs = [
-            ValueSpec(name="task_request", type=TASK_REQUEST_TYPE),
-            ValueSpec(name="task_result", type=TASK_RESULT_TYPE),
-            ValueSpec(name="task_exception", type=TASK_EXCEPTION_TYPE),
-            ValueSpec(name="task_state", type=TASK_STATE_TYPE),
-            ValueSpec(name="pipeline_state", type=PIPELINE_STATE_TYPE),
+            ValueSpec(name="task_request", type=TASK_REQUEST_TYPE, expire_after_call=state_expiration),
+            ValueSpec(name="task_result", type=TASK_RESULT_TYPE, expire_after_call=state_expiration),
+            ValueSpec(name="task_exception", type=TASK_EXCEPTION_TYPE, expire_after_call=state_expiration),
+            ValueSpec(name="task_state", type=TASK_STATE_TYPE, expire_after_call=state_expiration),
+            ValueSpec(name="pipeline_state", type=PIPELINE_STATE_TYPE, expire_after_call=state_expiration),
         ]
 
         self.register_builtin(run_pipeline, with_context=True, with_state=True)
@@ -78,7 +81,7 @@ class FlinkTasks(object):
         def send(*args, **kwargs):
             return PipelineBuilder().send(function, *args, **kwargs)
 
-        def to_task(args, kwargs, is_finally=False, parameters={}):
+        def to_task(args, kwargs, is_finally=False, parameters={}, is_exceptionally=False):
             parameters = {**defaults(), **parameters}
 
             if is_finally:
@@ -90,7 +93,7 @@ class FlinkTasks(object):
             task_id = parameters.pop('task_id') or _gen_id()
             args = _unpack_single_tuple_args(args)
 
-            return Task.from_fields(task_id, task_type, args, kwargs, is_finally=is_finally, **parameters)
+            return Task.from_fields(task_id, task_type, args, kwargs, is_finally=is_finally, is_exceptionally=is_exceptionally, **parameters)
 
         function.send = send
         function.to_task = to_task
@@ -200,7 +203,7 @@ class FlinkTasks(object):
         :param context: context object provided by Flink
         :param message: the task input protobuf message
         """
-        with TaskContext(context, self._egress_type_name, self._serialiser) as task_context:
+        with TaskContext(context, self._egress_type_name, self._egress_message_max_size, self._serialiser) as task_context:
 
             for handler in self._handlers:
                 task_input = handler.unpack(task_context, message)
@@ -215,7 +218,7 @@ class FlinkTasks(object):
 
                     except Exception as ex:
                         _log.error(f'Error invoking {task_context} - {ex}')
-                        self.fail(task_context, task_input, ex)
+                        await self.fail(task_context, task_input, ex)
                         
                     finally:
                         return
@@ -273,7 +276,7 @@ class FlinkTasks(object):
         args, kwargs, state = self._serialiser.deserialise_request(task_request)
         return args, kwargs, state
 
-    def send_result(self, context: TaskContext, task_request: TaskRequest, result, state=Ellipsis, delay: timedelta=None, cancellation_token: str = ""):
+    async def send_result(self, context: TaskContext, task_request: TaskRequest, result, state=Ellipsis, delay: timedelta=None, cancellation_token: str = ""):
         """
         Sends a result
         :param context: TaskContext
@@ -290,9 +293,9 @@ class FlinkTasks(object):
 
         self._serialiser.serialise_result(task_result, result, state)
 
-        self.emit_result(context, task_request, task_result, delay,  cancellation_token)
+        await self.emit_result(context, task_request, task_result, delay,  cancellation_token)
 
-    def fail(self, context, task_input, ex, delay: timedelta=None, cancellation_token: str = ""):
+    async def fail(self, context, task_input, ex, delay: timedelta=None, cancellation_token: str = ""):
         """
         Sends a failure
         :param context: TaskContext
@@ -302,7 +305,7 @@ class FlinkTasks(object):
         :param optional cancellation_token: a cancellation token to associate with this message
         """
         task_exception = _create_task_exception(task_input, ex)
-        self.emit_result(context, task_input, task_exception, delay,  cancellation_token)
+        await self.emit_result(context, task_input, task_exception, delay, cancellation_token)
 
     def get_pipeline(self, context):
         pipeline_protos = context.pipeline_state.pipeline
@@ -318,20 +321,26 @@ class FlinkTasks(object):
         except:
             return None
 
-    def emit_result(self, context, task_input, task_result, delay: timedelta=None, cancellation_token: str = None):
-        # copy over invocation id
+    async def emit_result(self, context, task_input, task_result, delay: timedelta=None, cancellation_token: str = None):
+        # copy over invocation id and notify we are about to emit a result
         if isinstance(task_result, (TaskResult, TaskException)):
             task_result.invocation_id = task_input.invocation_id
+            await self.events.notify_emit_result(context, task_result)
             
-        # either send a message to egress if reply_topic was specified
+        # send a message to egress if reply_topic was specified
         if task_input.HasField('reply_topic'):
-            context.send_egress_message(topic=task_input.reply_topic, value=task_result)
+            try:
+                context.send_egress_message(topic=task_input.reply_topic, value=task_result)
+            except MessageSizeExceeded as e:
+                _log.warning(f'Unable to send egress message - {e}')
+                context.send_egress_message(topic=task_input.reply_topic, value=_create_task_exception(task_input, e))
 
         # or call back to a particular flink function if reply_address was specified
         elif task_input.HasField('reply_address'):
             address, identifer = context.to_address_and_id(task_input.reply_address)
             context.send_message(address, identifer, task_result, delay, cancellation_token)
 
+        # otherwise call back to the caller (if there is one)
         elif isinstance(task_input, TaskRequest):
             address, caller_id = self._get_caller_address_and_id(context, task_input)
             if address is not None and caller_id is not None:
