@@ -3,6 +3,7 @@ from statefun_tasks.utils import _type_name
 from statefun_tasks.protobuf import pack_any
 from statefun_tasks.messages_pb2 import TaskRequest, TaskResult, TaskException, TaskActionRequest, TaskActionResult, TaskActionException
 
+from statefun import Type
 import traceback as tb
 
 
@@ -16,6 +17,15 @@ def flink_value_type_for(proto):
     raise ValueError(f'No Flink value type found for proto type {proto_type}')
 
 
+def add_flink_value_type_for(proto, value_type: Type):
+    proto_type = type(proto)
+
+    if proto_type in _VALUE_TYPE_MAP:
+        ValueError(f'{proto_type} already exists')
+
+    _VALUE_TYPE_MAP[proto_type] = value_type
+
+
 def _create_task_exception(task_input, ex, state=None):
     if isinstance(task_input, TaskActionRequest):
         return TaskActionException(
@@ -58,4 +68,7 @@ def _create_task_result(task_input, result=None, state=None):
     if result is not None:
         task_result.result.CopyFrom(pack_any(result))
 
+    if state is not None:
+        task_result.state.CopyFrom(state)
+
     return task_result
