@@ -57,7 +57,7 @@ class Task:
 
     @staticmethod
     def from_fields(task_id, task_type, task_args, task_kwargs, is_finally=None, namespace=None, worker_name=None,
-                    is_fruitful=None, retry_policy=None, display_name=None, is_wait=None, uid=None, **kwargs):
+                    is_fruitful=None, retry_policy=None, display_name=None, is_wait=None, uid=None, is_exceptionally=None, **kwargs):
         proto = TaskEntry(
             task_id=task_id,
             task_type=task_type,
@@ -69,7 +69,8 @@ class Task:
             retry_policy=retry_policy,
             display_name=display_name,
             is_wait=is_wait,
-            uid=uid if uid is not None else _gen_id()
+            uid=uid if uid is not None else _gen_id(),
+            is_exceptionally = is_exceptionally
         )
         return Task(proto, task_args, task_kwargs)
 
@@ -119,8 +120,18 @@ class Task:
         """
         return self._proto.is_finally
 
+    @property
+    def is_exceptionally(self):
+        """
+        Whether this task is an exceptionally task
+        """
+        return self._proto.is_exceptionally
+
     @property
     def is_fruitful(self):
+        """
+        Whether this task is returns a result (fruitful) or not (void)
+        """
         return self._proto.is_fruitful
 
     @is_fruitful.setter
@@ -211,8 +222,8 @@ class Task:
 
         return self.task_id, self.task_type, self._args, self._kwargs
 
-    def mark_complete(self):
-        self._proto.complete = True
+    def mark_complete(self, value=True):
+        self._proto.complete = value
 
     def is_complete(self):
         return self._proto.complete
@@ -236,12 +247,13 @@ class Task:
 
 
 class Group:
-    __slots__ = ('group_id', 'max_parallelism', 'is_wait', '_group')
+    __slots__ = ('group_id', 'max_parallelism', 'is_wait', '_group', 'return_exceptions')
 
-    def __init__(self, group_id, max_parallelism=None, is_wait=None):
+    def __init__(self, group_id, max_parallelism=None, is_wait=None, return_exceptions=False):
         self.group_id = group_id
         self.max_parallelism = max_parallelism
         self.is_wait = is_wait
+        self.return_exceptions = return_exceptions
         self._group = []
 
     def add_to_group(self, tasks):
@@ -261,13 +273,24 @@ class Group:
 
     def to_proto(self, serialiser) -> PipelineEntry:
         group = [Pipeline(entries=[entry.to_proto(serialiser) for entry in entries]) for entries in self._group]
-        proto = GroupEntry(group_id=self.group_id, group=group, max_parallelism=self.max_parallelism, is_wait=self.is_wait)
+
+        proto = GroupEntry(
+            group_id=self.group_id, 
+            group=group, 
+            max_parallelism=self.max_parallelism, 
+            is_wait=self.is_wait,
+            return_exceptions=self.return_exceptions)
 
         return PipelineEntry(group_entry=proto)
 
     @staticmethod
     def from_proto(proto: PipelineEntry):
-        group = Group(group_id=proto.group_entry.group_id, max_parallelism=proto.group_entry.max_parallelism, is_wait=proto.group_entry.is_wait)
+        group = Group(
+            group_id=proto.group_entry.group_id, 
+            max_parallelism=proto.group_entry.max_parallelism, 
+            is_wait=proto.group_entry.is_wait,
+            return_exceptions=proto.group_entry.return_exceptions)
+            
         stack = deque([(proto.group_entry.group, group)])  
 
         while len(stack) > 0:
@@ -278,7 +301,13 @@ class Group:
 
                 for proto in pipeline.entries:
                     if proto.HasField('group_entry'):
-                        stack_group = Group(group_id=proto.group_entry.group_id, max_parallelism=proto.group_entry.max_parallelism, is_wait=proto.group_entry.is_wait)
+
+                        stack_group = Group(
+                            group_id=proto.group_entry.group_id, 
+                            max_parallelism=proto.group_entry.max_parallelism, 
+                            is_wait=proto.group_entry.is_wait,
+                            return_exceptions=proto.group_entry.return_exceptions)
+                        
                         entries.append(stack_group)
                         stack.append((proto.group_entry.group, stack_group))
                     else:
@@ -336,3 +365,9 @@ class ProtobufSerialisable(ABC):
     @abstractmethod
     def to_proto(serialiser):
         pass
+
+
+class MessageSizeExceeded(Exception):
+    def __init__(self):
+        super().__init__()
+
